package explorviz.live_trace_processing.filter.reduction.summarization;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractAfterFailedEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public class AssociativityFilter extends AbstractReductionFilter {

	public AssociativityFilter(final IPipeReceiver receiver) {
		super(receiver, "AssoziationFilter");
	}

	@Override
	protected Trace reduceTrace(final Trace trace) {
		final List<AbstractEventRecord> newTraceEvents = new ArrayList<AbstractEventRecord>();
		String caller = null;
		int count = 0;
		final Stack<Integer> stack = new Stack<Integer>();
		final Stack<String> callerStack = new Stack<String>();
		for (final AbstractEventRecord record : trace.getTraceEvents()) {
			if (record instanceof AbstractBeforeEventRecord) {
				count++;
				final AbstractBeforeEventRecord abstractBeforeEventRecord = (AbstractBeforeEventRecord) record;
				final String signature = abstractBeforeEventRecord.getOperationSignature();
				final String callee = getClazzFullQName(signature);
				if (!callee.equals(caller)) {
					newTraceEvents.add(record);
				} else {
					stack.push(new Integer(count));
				}
				caller = callee;
				callerStack.push(caller);
			} else if ((record instanceof AbstractAfterEventRecord)
					|| (record instanceof AbstractAfterFailedEventRecord)) {
				callerStack.pop();
				if (!callerStack.isEmpty()) {
					caller = callerStack.peek();
				}
				if (!stack.isEmpty() && (count == stack.peek())) {
					stack.pop();
				} else {
					newTraceEvents.add(record);
				}
				count--;
			} else {
				newTraceEvents.add(record);
			}
		}
		return new Trace(newTraceEvents, true);
	}

	private String getClazzFullQName(final String operationSignatureStr) {
		final int brace = operationSignatureStr.indexOf('(');
		final String signatureWithoutArguments = operationSignatureStr.substring(0, brace);
		final int lastWhiteSpace = signatureWithoutArguments.lastIndexOf(' ');
		final int lastDot = signatureWithoutArguments.lastIndexOf('.');
		if (lastDot == -1) {
			return "";
		}
		final String fullQClazzName = signatureWithoutArguments.substring(lastWhiteSpace + 1,
				lastDot);

		if (fullQClazzName.indexOf("$") > 0) {
			return fullQClazzName.substring(0, fullQClazzName.indexOf("$"));
		} else {
			return fullQClazzName;
		}
	}

}
